import re
import argparse

def tag_it(tag: str, string: str):
	if tag == "img":
		value = f"<img src='{string}'></img>"
		return value
	if tag == "br":
		value = "<br>"
		return value
	if tag == "a":
		link = re.findall(r'\[(.*?)\]', string)
		if link:
			link = link[0]
			string = string.split('[')[0]
			value = f"<{tag} href='{link}'>{string}</{tag}>"
			return value
	else:
		value = f"<{tag}>{string}</{tag}>"
		return value

def main(filename: str):
	data = []
	with open(filename) as f:
		file_to_list = f.read().splitlines()
		
		for file_text in file_to_list:
			tag = file_text.split(':')[0]
			string = file_text.replace(tag + ":" , '')
			string = tag_it(tag, string)
			
			data.append(string)
			
		return data
		
if __name__=="__main__":
	
	parser = argparse.ArgumentParser()
	parser.add_argument('-f', '--filename', help='The file contains plain text you just wrote')
	args = parser.parse_args()
	if args.filename:
		filename = args.filename
	else:
		filename = input('filename: str -> ')
	data = main(filename)
	data = "\n".join([v for v in data])
	f = open(filename.split('.')[0] + ".html", "w")
	f.write(data)


import mimetypes

import requests

from .exceptions import TelegraphException


def upload_file(f):
    """ Upload file to Telegra.ph's servers. Returns a list of links.
        Allowed only .jpg, .jpeg, .png, .gif and .mp4 files.

    :param f: filename or file-like object.
    :type f: file, str or list
    """
    with FilesOpener(f) as files:
        response = requests.post(
            'https://telegra.ph/upload',
            files=files
        ).json()

    if isinstance(response, list):
        error = response[0].get('error')
    else:
        error = response.get('error')

    if error:
        raise TelegraphException(error)

    return [i['src'] for i in response]


class FilesOpener(object):
    def __init__(self, paths, key_format='file{}'):
        if not isinstance(paths, list):
            paths = [paths]

        self.paths = paths
        self.key_format = key_format
        self.opened_files = []

    def __enter__(self):
        return self.open_files()

    def __exit__(self, type, value, traceback):
        self.close_files()

    def open_files(self):
        self.close_files()

        files = []

        for x, file_or_name in enumerate(self.paths):
            name = ''
            if isinstance(file_or_name, tuple) and len(file_or_name) >= 2:
                name = file_or_name[1]
                file_or_name = file_or_name[0]

            if hasattr(file_or_name, 'read'):
                f = file_or_name

                if hasattr(f, 'name'):
                    filename = f.name
                else:
                    filename = name
            else:
                filename = file_or_name
                f = open(filename, 'rb')
                self.opened_files.append(f)

            mimetype = mimetypes.MimeTypes().guess_type(filename)[0]

            files.append(
                (self.key_format.format(x), ('file{}'.format(x), f, mimetype))
            )

        return files

    def close_files(self):
        for f in self.opened_files:
            f.close()

        self.opened_files = []



# -*- coding: utf-8 -*-
import json
import requests
from .exceptions import TelegraphException
from .utils import html_to_nodes, nodes_to_html


class TelegraphApi(object):
    __slots__ = ('access_token', 'session')
    def __init__(self, access_token=None):
        self.access_token = access_token
        self.session = requests.Session()

    def method(self, method, values=None, path=''):
        values = values.copy() if values is not None else {}

        if 'access_token' not in values and self.access_token:
            values['access_token'] = self.access_token

        response = self.session.post(
            'https://api.telegra.ph/{}/{}'.format(method, path),
            values
        ).json()

        if response.get('ok'):
            return response['result']

        raise TelegraphException(response.get('error'))


class Telegraph(object):
    __slots__ = ('_telegraph',)
    def __init__(self, access_token=None):
        self._telegraph = TelegraphApi(access_token)

    def get_access_token(self):
        return self._telegraph.access_token

    def create_account(self, short_name, author_name=None, author_url=None,
                       replace_token=True):
        response = self._telegraph.method('createAccount', values={
            'short_name': short_name,
            'author_name': author_name,
            'author_url': author_url
        })
        if replace_token:
            self._telegraph.access_token = response.get('access_token')
        return response

    def edit_account_info(self, short_name=None, author_name=None,
                          author_url=None):
        return self._telegraph.method('editAccountInfo', values={
            'short_name': short_name,
            'author_name': author_name,
            'author_url': author_url
        })

    def revoke_access_token(self):
        response = self._telegraph.method('revokeAccessToken')

        self._telegraph.access_token = response.get('access_token')

        return response

    def get_page(self, path, return_content=True, return_html=True):
        response = self._telegraph.method('getPage', path=path, values={
            'return_content': return_content
        })

        if return_content and return_html:
            response['content'] = nodes_to_html(response['content'])

        return response

    def create_page(self, title, content=None, html_content=None,
                    author_name=None, author_url=None, return_content=False):
        if content is None:
            content = html_to_nodes(html_content)

        content_json = json.dumps(content, ensure_ascii=False)

        return self._telegraph.method('createPage', values={
            'title': title,
            'author_name': author_name,
            'author_url': author_url,
            'content': content_json,
            'return_content': return_content
        })

    def edit_page(self, path, title, content=None, html_content=None,
                  author_name=None, author_url=None, return_content=False):
        if content is None:
            content = html_to_nodes(html_content)

        content_json = json.dumps(content, ensure_ascii=False)

        return self._telegraph.method('editPage', path=path, values={
            'title': title,
            'author_name': author_name,
            'author_url': author_url,
            'content': content_json,
            'return_content': return_content
        })

    def get_account_info(self, fields=None):
        return self._telegraph.method('getAccountInfo', {
            'fields': json.dumps(fields, ensure_ascii=False) if fields else None
        })

    def get_page_list(self, offset=0, limit=50):
        return self._telegraph.method('getPageList', {
            'offset': offset,
            'limit': limit
        })

    def get_views(self, path, year=None, month=None, day=None, hour=None):
        return self._telegraph.method('getViews', path=path, values={
            'year': year,
            'month': month,
            'day': day,
            'hour': hour
        })



# -*- coding: utf-8 -*-

"""
@author: python273
@contact: https://python273.pw
@license MIT License, see LICENSE file

Copyright (C) 2018
"""

__author__ = 'python273'
__version__ = '1.4.1'

from .api import Telegraph, TelegraphException
from .upload import upload_file


# -*- coding: utf-8 -*-
import re

from .exceptions import NotAllowedTag, InvalidHTML

try:  # python 3.x
    from html.parser import HTMLParser
    from html.entities import name2codepoint
    from html import escape

    basestring = str

except ImportError:  # python 2.x
    from HTMLParser import HTMLParser
    from htmlentitydefs import name2codepoint
    from cgi import escape

    chr = unichr


RE_WHITESPACE = re.compile(r'(\s+)', re.UNICODE)


ALLOWED_TAGS = {
    'a', 'aside', 'b', 'blockquote', 'br', 'code', 'em', 'figcaption', 'figure',
    'h3', 'h4', 'hr', 'i', 'iframe', 'img', 'li', 'ol', 'p', 'pre', 's',
    'strong', 'u', 'ul', 'video'
}

VOID_ELEMENTS = {
    'area', 'base', 'br', 'col', 'embed', 'hr', 'img', 'input', 'keygen',
    'link', 'menuitem', 'meta', 'param', 'source', 'track', 'wbr'
}

BLOCK_ELEMENTS = {
    'address', 'article', 'aside', 'blockquote', 'canvas', 'dd', 'div', 'dl',
    'dt', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'h1', 'h2',
    'h3', 'h4', 'h5', 'h6', 'header', 'hgroup', 'hr', 'li', 'main', 'nav',
    'noscript', 'ol', 'output', 'p', 'pre', 'section', 'table', 'tfoot', 'ul',
    'video'
}


class HtmlToNodesParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)

        self.nodes = []

        self.current_nodes = self.nodes
        self.parent_nodes = []

    def add_str_node(self, s):
        if not s:
            return

        if self.current_nodes and isinstance(self.current_nodes[-1], basestring):
            self.current_nodes[-1] += s
        else:
            self.current_nodes.append(s)

    def handle_starttag(self, tag, attrs_list):
        if tag not in ALLOWED_TAGS:
            raise NotAllowedTag('%s tag is not allowed' % tag)

        node = {'tag': tag}

        if attrs_list:
            attrs = {}

            for attr, value in attrs_list:
                attrs[attr] = value

            node['attrs'] = attrs

        self.current_nodes.append(node)

        if tag not in VOID_ELEMENTS:
            self.parent_nodes.append(self.current_nodes)
            self.current_nodes = node['children'] = []

    def handle_endtag(self, tag):
        if tag in VOID_ELEMENTS:
            return

        if not len(self.parent_nodes):
            raise InvalidHTML('"{}" missing start tag'.format(
                tag
            ))

        self.current_nodes = self.parent_nodes.pop()

        last_node = self.current_nodes[-1]

        if last_node['tag'] != tag:
            raise InvalidHTML('"{}" tag closed instead of "{}"'.format(
                tag, last_node['tag']
            ))

        if not last_node['children']:
            last_node.pop('children')

    def handle_data(self, data):
        self.add_str_node(data)

    def handle_entityref(self, name):
        self.add_str_node(chr(name2codepoint[name]))

    def handle_charref(self, name):
        if name.startswith('x'):
            c = chr(int(name[1:], 16))
        else:
            c = chr(int(name))

        self.add_str_node(c)

    def get_nodes(self):
        if self.parent_nodes:
            not_closed_tag = self.parent_nodes[-1][-1]['tag']
            raise InvalidHTML('"{}" tag is not closed'.format(not_closed_tag))

        return self.nodes


def clear_whitespace_nodes(nodes, last_text_node=None):
    """

    :param nodes:
    :type nodes: list
    :param last_text_node:
    :type last_text_node: basestring
    :return: list
    """
    # TODO: probably possible to move to html parser

    stack = []
    current_nodes = nodes[:]

    new_nodes = []
    new_children = new_nodes

    while True:
        if current_nodes:
            node = current_nodes.pop(0)

            if type(node) is dict:
                is_block_element = node['tag'] in BLOCK_ELEMENTS
                if is_block_element:
                    last_text_node = None

                new_children.append(node)

                node_children = node.get('children')

                if node_children:
                    stack.append((current_nodes, new_children))
                    current_nodes = node_children
                    new_children = []
                    node['children'] = new_children
            else:
                node = RE_WHITESPACE.sub(' ', node)

                if last_text_node is None or last_text_node.endswith(' '):
                    node = node.lstrip(' ')

                if node:
                    last_text_node = node
                    new_children.append(node)
                else:
                    last_text_node = None

        if not current_nodes:
            if stack:
                current_nodes, new_children = stack.pop()
            else:
                break

    return new_nodes, last_text_node


def html_to_nodes(html_content):
    parser = HtmlToNodesParser()
    parser.feed(html_content)

    nodes = parser.get_nodes()
    nodes, _ = clear_whitespace_nodes(nodes)
    return nodes


def nodes_to_html(nodes):
    html_content = []

    stack = []
    tags_stack = []
    current_nodes = nodes[:]

    while True:
        if current_nodes:
            node = current_nodes.pop(0)

            if type(node) is dict:
                tags_stack.append(node['tag'])

                attrs = node.get('attrs')

                if attrs:
                    attrs_str = ['']

                    for attr, value in attrs.items():
                        attrs_str.append('{}="{}"'.format(attr, escape(value)))
                else:
                    attrs_str = []

                html_content.append('<{}{}>'.format(
                    node['tag'],
                    ' '.join(attrs_str)
                ))

                children = node.get('children', [])
                stack.append(current_nodes)
                current_nodes = children
            else:
                html_content.append(escape(node))

        if not current_nodes:
            if tags_stack:
                closed_tag = tags_stack.pop()

                last_el = html_content[-1]

                if closed_tag in VOID_ELEMENTS and \
                   last_el.startswith('<{}'.format(closed_tag)) and \
                   not last_el.endswith('/>'):

                    html_content[-1] = last_el[:-1] + '/>'
                else:
                    html_content.append('</{}>'.format(closed_tag))

            if stack:
                current_nodes = stack.pop()
            else:
                break

    return ''.join(html_content)



from setuptools import setup

long_description = open('README.md').read()
setup(
	url='https://github.com/aN4ksaL4y/mytelegraph',
	name='mytelegraph',
	author='Cingkariak Sawah',
	author_email='fajrim228@gmail.com',
	description='a tiny helper for posting to a Telegra.ph page.',
	long_description = long_description,
	long_description_content_type='text/markdown',
	version='0.0.3', 
	packages=['telegraph'], 
	scripts=['bin/posting']
)



#! /usr/bin/env python3

import os
import sys
import json
import argparse
import requests 
from telegraph.api import Telegraph
from telegraph.conf import token
from telegraph.utils import html_to_nodes

__author__="Muhammad Al Fajri"
__email__="fajrim228@gmail.com"
__telegram__="https://t.me/ini_peninggi_badan"

def create_page(path, values: dict = None):
	"""
	Simple!
	"""
	try:
		r = requests.post(f'https://api.telegra.ph/{path}', params=values)
		return r.json()
	except Exception as e:
		raise e
	
def get_all_page():
	'''
	Place a description
	'''
	T = Telegraph(access_token = token)
	response = T.get_page_list()
	return response

def create_token(author: str, short_name: str):
	'''
	Place a description
	'''
	data = {
		'short_name':short_name,
		'author_name':author
	}
	r = requests.post('https://api.telegra.ph/createAccount', params=data)
	return r.text
	
def main(
		string_input: str = None,
		args_filename: str = None,
		args_title: str = None,
		args_author: str = None,
		args_author_url: str = None,
		args_page: str = None
	):
	global __author__
	author_name = __author__
	author_url = "https://t.me/" + __telegram__
	'''
	* this is the main script, you only need to call this one.
	* take an argument (text/html) for now, default to None

	'''
	if args_page:
		print(get_all_page())
		sys.exit(0)
	if args_filename:
		html_file = args_filename
	elif not args_filename:
		html_file = os.getenv('HTML_FILE')
	else:
		return None
	if args_title:
		title = args_title
	else:
		title = html_file.replace('-', ' ')
	if args_author:
		author_name = args_author
	else:
		author_name = author_name
	if args_author_url:
		author_url = args_author_url
	else:
		author_url = author_url
	a = open(html_file).read()
	if string_input:
		content = html_to_nodes(string_input)
	else:
		content = html_to_nodes(a)
	content_json = json.dumps(content, ensure_ascii=False)
	data = {
		'access_token':token,
		'title': title,
		'author_name': author_name,
		'author_url': author_url,
		'content': content_json,
		'return_content': True
	}
	try:
		return (
			create_page('createPage', data)['result']
		)
	except Exception as e:
		return None
